﻿namespace DotSDL.Input.Keyboard {
    /// <summary>
    /// Contains constants used for scancode conversion.
    /// </summary>
    internal static class KeycodeConstants {
        internal const int ScancodeMask = 1 << 30;
    }

    /// <summary>
    /// Contains a list of keycodes used by SDL2.
    /// </summary>
    public enum Keycode {
        Unknown = 0,
        Return = '\r',
        Escape = 0x1B,
        Backspace = '\b',
        Tab = '\t',
        Space = ' ',
        Exclaim = '!',
        QuoteDbl = '"',
        Hash = '#',
        Percent = '%',
        Dollar = '$',
        Ampersand = '&',
        Quote = '\'',
        LeftParen = '(',
        RightParen = ')',
        Asterisk = '*',
        Plus = '+',
        Comma = ',',
        Minus = '-',
        Period = '.',
        Slash = '/',
        Num0 = '0',
        Num1 = '1',
        Num2 = '2',
        Num3 = '3',
        Num4 = '4',
        Num5 = '5',
        Num6 = '6',
        Num7 = '7',
        Num8 = '8',
        Num9 = '9',
        Colon = ':',
        Semicolon = ';',
        Less = '<',
        Equals = '=',
        Greater = '>',
        Question = '?',
        At = '@',
        LeftBracket = '[',
        Backslash = '\\',
        RightBracket = ']',
        Caret = '^',
        Underscore = '_',
        Backquote = '`',
        A = 'a',
        B = 'b',
        C = 'c',
        D = 'd',
        E = 'e',
        F = 'f',
        G = 'g',
        H = 'h',
        I = 'i',
        J = 'j',
        K = 'k',
        L = 'l',
        M = 'm',
        N = 'n',
        O = 'o',
        P = 'p',
        Q = 'q',
        R = 'r',
        S = 's',
        T = 't',
        U = 'u',
        V = 'v',
        W = 'w',
        X = 'x',
        Y = 'y',
        Z = 'z',
        CapsLock = Scancode.CapsLock | KeycodeConstants.ScancodeMask,
        F1 = Scancode.F1 | KeycodeConstants.ScancodeMask,
        F2 = Scancode.F2 | KeycodeConstants.ScancodeMask,
        F3 = Scancode.F3 | KeycodeConstants.ScancodeMask,
        F4 = Scancode.F4 | KeycodeConstants.ScancodeMask,
        F5 = Scancode.F5 | KeycodeConstants.ScancodeMask,
        F6 = Scancode.F6 | KeycodeConstants.ScancodeMask,
        F7 = Scancode.F7 | KeycodeConstants.ScancodeMask,
        F8 = Scancode.F8 | KeycodeConstants.ScancodeMask,
        F9 = Scancode.F9 | KeycodeConstants.ScancodeMask,
        F10 = Scancode.F10 | KeycodeConstants.ScancodeMask,
        F11 = Scancode.F11 | KeycodeConstants.ScancodeMask,
        F12 = Scancode.F12 | KeycodeConstants.ScancodeMask,
        PrintScreen = Scancode.PrintScreen | KeycodeConstants.ScancodeMask,
        ScrollLock = Scancode.ScrollLock | KeycodeConstants.ScancodeMask,
        Pause = Scancode.Pause | KeycodeConstants.ScancodeMask,
        Insert = Scancode.Insert | KeycodeConstants.ScancodeMask,
        Home = Scancode.Home | KeycodeConstants.ScancodeMask,
        PageUp = Scancode.PageUp | KeycodeConstants.ScancodeMask,
        Delete = 0x7F,
        End = Scancode.End | KeycodeConstants.ScancodeMask,
        PageDown = Scancode.PageDown | KeycodeConstants.ScancodeMask,
        Right = Scancode.Right | KeycodeConstants.ScancodeMask,
        Left = Scancode.Left | KeycodeConstants.ScancodeMask,
        Down = Scancode.Down | KeycodeConstants.ScancodeMask,
        Up = Scancode.Up | KeycodeConstants.ScancodeMask,
        NumLockClear = Scancode.NumLockClear | KeycodeConstants.ScancodeMask,
        NumPadDivide = Scancode.NumPadDivide | KeycodeConstants.ScancodeMask,
        NumPadMuliply = Scancode.NumPadMultiply | KeycodeConstants.ScancodeMask,
        NumPadMinus = Scancode.NumPadMinus | KeycodeConstants.ScancodeMask,
        NumPadPlus = Scancode.NumPadPlus | KeycodeConstants.ScancodeMask,
        NumPadEnter = Scancode.NumPadEnter | KeycodeConstants.ScancodeMask,
        NumPad1 = Scancode.NumPad1 | KeycodeConstants.ScancodeMask,
        NumPad2 = Scancode.NumPad2 | KeycodeConstants.ScancodeMask,
        NumPad3 = Scancode.NumPad3 | KeycodeConstants.ScancodeMask,
        NumPad4 = Scancode.NumPad4 | KeycodeConstants.ScancodeMask,
        NumPad5 = Scancode.NumPad5 | KeycodeConstants.ScancodeMask,
        NumPad6 = Scancode.NumPad6 | KeycodeConstants.ScancodeMask,
        NumPad7 = Scancode.NumPad7 | KeycodeConstants.ScancodeMask,
        NumPad8 = Scancode.NumPad8 | KeycodeConstants.ScancodeMask,
        NumPad9 = Scancode.NumPad9 | KeycodeConstants.ScancodeMask,
        NumPad0 = Scancode.NumPad0 | KeycodeConstants.ScancodeMask,
        NumPadPeriod = Scancode.NumPadPeriod | KeycodeConstants.ScancodeMask,
        Application = Scancode.Application | KeycodeConstants.ScancodeMask,
        Power = Scancode.Power | KeycodeConstants.ScancodeMask,
        NumPadEquals = Scancode.NumPadEquals | KeycodeConstants.ScancodeMask,
        F13 = Scancode.F13 | KeycodeConstants.ScancodeMask,
        F14 = Scancode.F14 | KeycodeConstants.ScancodeMask,
        F15 = Scancode.F15 | KeycodeConstants.ScancodeMask,
        F16 = Scancode.F16 | KeycodeConstants.ScancodeMask,
        F17 = Scancode.F17 | KeycodeConstants.ScancodeMask,
        F18 = Scancode.F18 | KeycodeConstants.ScancodeMask,
        F19 = Scancode.F19 | KeycodeConstants.ScancodeMask,
        F20 = Scancode.F20 | KeycodeConstants.ScancodeMask,
        F21 = Scancode.F21 | KeycodeConstants.ScancodeMask,
        F22 = Scancode.F22 | KeycodeConstants.ScancodeMask,
        F23 = Scancode.F23 | KeycodeConstants.ScancodeMask,
        F24 = Scancode.F24 | KeycodeConstants.ScancodeMask,
        Execute = Scancode.Execute | KeycodeConstants.ScancodeMask,
        Help = Scancode.Help | KeycodeConstants.ScancodeMask,
        Menu = Scancode.Menu | KeycodeConstants.ScancodeMask,
        Select = Scancode.Select | KeycodeConstants.ScancodeMask,
        Stop = Scancode.Stop | KeycodeConstants.ScancodeMask,
        Again = Scancode.Again | KeycodeConstants.ScancodeMask,
        Undo = Scancode.Undo | KeycodeConstants.ScancodeMask,
        Cut = Scancode.Cut | KeycodeConstants.ScancodeMask,
        Copy = Scancode.Copy | KeycodeConstants.ScancodeMask,
        Paste = Scancode.Paste | KeycodeConstants.ScancodeMask,
        Find = Scancode.Find | KeycodeConstants.ScancodeMask,
        Mute = Scancode.Mute | KeycodeConstants.ScancodeMask,
        VolumeUp = Scancode.VolumeUp | KeycodeConstants.ScancodeMask,
        VolumeDown = Scancode.VolumeDown | KeycodeConstants.ScancodeMask,
        NumPadComma = Scancode.NumPadComma | KeycodeConstants.ScancodeMask,
        NumPadEqualsAs400 = Scancode.NumPadEqualsAs400 | KeycodeConstants.ScancodeMask,
        AltErase = Scancode.AltErase | KeycodeConstants.ScancodeMask,
        SysReq = Scancode.SysReq | KeycodeConstants.ScancodeMask,
        Cancel = Scancode.Cancel | KeycodeConstants.ScancodeMask,
        Clear = Scancode.Clear | KeycodeConstants.ScancodeMask,
        Prior = Scancode.Prior | KeycodeConstants.ScancodeMask,
        Return2 = Scancode.Return2 | KeycodeConstants.ScancodeMask,
        Separator = Scancode.Separator | KeycodeConstants.ScancodeMask,
        Out = Scancode.Out | KeycodeConstants.ScancodeMask,
        Oper = Scancode.Oper | KeycodeConstants.ScancodeMask,
        ClearAgain = Scancode.ClearAgain | KeycodeConstants.ScancodeMask,
        CrSel = Scancode.CrSel | KeycodeConstants.ScancodeMask,
        ExSel = Scancode.ExSel | KeycodeConstants.ScancodeMask,
        NumPad00 = Scancode.Numpad00 | KeycodeConstants.ScancodeMask,
        NumPad000 = Scancode.Numpad000 | KeycodeConstants.ScancodeMask,
        ThousandsSeparator = Scancode.ThousandsSeparator | KeycodeConstants.ScancodeMask,
        DecimalSeparator = Scancode.DecimalSeparator | KeycodeConstants.ScancodeMask,
        CurrencyUnit = Scancode.CurrencyUnit | KeycodeConstants.ScancodeMask,
        CurrencySubunit = Scancode.CurrencySubunit | KeycodeConstants.ScancodeMask,
        NumPadLeftParen = Scancode.NumPadLeftParen | KeycodeConstants.ScancodeMask,
        NumPadRightParen = Scancode.NumPadRightParen | KeycodeConstants.ScancodeMask,
        NumPadLeftBrace = Scancode.NumPadLeftBrace | KeycodeConstants.ScancodeMask,
        NumPadRightBrace = Scancode.NumPadRightBrace | KeycodeConstants.ScancodeMask,
        NumPadTab = Scancode.NumPadTab | KeycodeConstants.ScancodeMask,
        NumPadBackspace = Scancode.NumPadBackspace | KeycodeConstants.ScancodeMask,
        NumPadA = Scancode.NumPadA | KeycodeConstants.ScancodeMask,
        NumPadB = Scancode.NumPadB | KeycodeConstants.ScancodeMask,
        NumPadC = Scancode.NumPadC | KeycodeConstants.ScancodeMask,
        NumPadD = Scancode.NumPadD | KeycodeConstants.ScancodeMask,
        NumPadE = Scancode.NumPadE | KeycodeConstants.ScancodeMask,
        NumPadF = Scancode.NumPadF | KeycodeConstants.ScancodeMask,
        NumPadXor = Scancode.NumPadXor | KeycodeConstants.ScancodeMask,
        NumPadPower = Scancode.NumPadPower | KeycodeConstants.ScancodeMask,
        NumPadPercent = Scancode.NumPadPercent | KeycodeConstants.ScancodeMask,
        NumPadLess = Scancode.NumPadLess | KeycodeConstants.ScancodeMask,
        NumPadGreater = Scancode.NumPadGreater | KeycodeConstants.ScancodeMask,
        NumPadAmpersand = Scancode.NumPadAmpersand | KeycodeConstants.ScancodeMask,
        NumPadDblAmpersand = Scancode.NumPadDblAmpersand | KeycodeConstants.ScancodeMask,
        NumPadVerticalBar = Scancode.NumPadVerticalBar | KeycodeConstants.ScancodeMask,
        NumPadDblVerticalBar = Scancode.NumPadDblVerticalBar | KeycodeConstants.ScancodeMask,
        NumPadColon = Scancode.NumPadColon | KeycodeConstants.ScancodeMask,
        NumPadHash = Scancode.NumPadHash | KeycodeConstants.ScancodeMask,
        NumPadSpace = Scancode.NumPadSpace | KeycodeConstants.ScancodeMask,
        NumPadAt = Scancode.NumPadAt | KeycodeConstants.ScancodeMask,
        NumPadExclaim = Scancode.NumPadExclam | KeycodeConstants.ScancodeMask,
        NumPadMemStory = Scancode.NumPadMemStore | KeycodeConstants.ScancodeMask,
        NumPadMemRecall = Scancode.NumPadMemRecall | KeycodeConstants.ScancodeMask,
        NumPadMemClear = Scancode.NumPadMemClear | KeycodeConstants.ScancodeMask,
        NumPadMemAdd = Scancode.NumPadMemAdd | KeycodeConstants.ScancodeMask,
        NumPadMemSubtract = Scancode.NumPadMemSubtract | KeycodeConstants.ScancodeMask,
        NumPadMemMultiply = Scancode.NumPadMemMultiply | KeycodeConstants.ScancodeMask,
        NumPadMemDivide = Scancode.NumPadMemDivide | KeycodeConstants.ScancodeMask,
        NumPadPlusMinus = Scancode.NumPadPlusMinus | KeycodeConstants.ScancodeMask,
        NumPadClear = Scancode.NumPadClear | KeycodeConstants.ScancodeMask,
        NumPadClearEntry = Scancode.NumPadClearEntry | KeycodeConstants.ScancodeMask,
        NumPadBinary = Scancode.NumPadBinary | KeycodeConstants.ScancodeMask,
        NumPadOctal = Scancode.NumPadOctal | KeycodeConstants.ScancodeMask,
        NumPadDecimal = Scancode.NumPadDecimal | KeycodeConstants.ScancodeMask,
        NumPadHexadecimal = Scancode.NumPadHexadecimal | KeycodeConstants.ScancodeMask,
        LCtrl = Scancode.LCtrl | KeycodeConstants.ScancodeMask,
        LShift = Scancode.LShift | KeycodeConstants.ScancodeMask,
        LAlt = Scancode.LAlt | KeycodeConstants.ScancodeMask,
        LGui = Scancode.LGui | KeycodeConstants.ScancodeMask,
        RCtrl = Scancode.RCtrl | KeycodeConstants.ScancodeMask,
        RShift = Scancode.RShift | KeycodeConstants.ScancodeMask,
        RAlt = Scancode.RAlt | KeycodeConstants.ScancodeMask,
        RGui = Scancode.RGui | KeycodeConstants.ScancodeMask,
        Mode = Scancode.Mode | KeycodeConstants.ScancodeMask,
        AudioNext = Scancode.AudioNext | KeycodeConstants.ScancodeMask,
        AudioPrev = Scancode.AudioPrev | KeycodeConstants.ScancodeMask,
        AudioStop = Scancode.AudioStop | KeycodeConstants.ScancodeMask,
        AudioPlay = Scancode.AudioPlay | KeycodeConstants.ScancodeMask,
        AudioMute = Scancode.AudioMute | KeycodeConstants.ScancodeMask,
        MediaSelect = Scancode.MediaSelect | KeycodeConstants.ScancodeMask,
        Web = Scancode.Web | KeycodeConstants.ScancodeMask,
        Mail = Scancode.Mail | KeycodeConstants.ScancodeMask,
        Calculator = Scancode.Calculator | KeycodeConstants.ScancodeMask,
        Computer = Scancode.Computer | KeycodeConstants.ScancodeMask,
        AcSearch = Scancode.AcSearch | KeycodeConstants.ScancodeMask,
        AcHome = Scancode.AcHome | KeycodeConstants.ScancodeMask,
        AcBack = Scancode.AcBack | KeycodeConstants.ScancodeMask,
        AcForward = Scancode.AcForward | KeycodeConstants.ScancodeMask,
        AcStop = Scancode.AcStop | KeycodeConstants.ScancodeMask,
        AcRefresh = Scancode.AcRefresh | KeycodeConstants.ScancodeMask,
        AcBookmarks = Scancode.AcBookmarks | KeycodeConstants.ScancodeMask,
        BrightnessDown = Scancode.BrightnessDown | KeycodeConstants.ScancodeMask,
        BrightnessUp = Scancode.BrightnessUp | KeycodeConstants.ScancodeMask,
        DisplaySwitch = Scancode.DisplaySwitch | KeycodeConstants.ScancodeMask,
        KbdIllumToggle = Scancode.KbdIllumToggle | KeycodeConstants.ScancodeMask,
        KbdIllumDown = Scancode.KbdIllumDown | KeycodeConstants.ScancodeMask,
        KbdIllumUp = Scancode.KbdIllumUp | KeycodeConstants.ScancodeMask,
        Eject = Scancode.Eject | KeycodeConstants.ScancodeMask,
        Sleep = Scancode.Sleep | KeycodeConstants.ScancodeMask
    }
}
